"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.loadConfiguration = exports.globFiles = exports.possibleJSONStringToArray = void 0;
const node_assert_1 = require("node:assert");
const debug_1 = __importDefault(require("debug"));
const camelcase_1 = __importDefault(require("camelcase"));
const nconf_1 = __importDefault(require("nconf"));
const dotenv_1 = __importDefault(require("dotenv"));
const deepmerge_1 = __importDefault(require("@fastify/deepmerge"));
const promises_1 = __importDefault(require("node:fs/promises"));
const glob_1 = require("glob");
const path_1 = __importDefault(require("path"));
const secure_json_parse_1 = __importDefault(require("secure-json-parse"));
const debug = (0, debug_1.default)('ms-conf');
const { hasOwnProperty } = Object.prototype;
const { isArray } = Array;
const { env } = process;
const verbose = hasOwnProperty.call(env, 'DOTENV_NOT_SILENT') === false;
const cwd = process.cwd();
const merge = (0, deepmerge_1.default)({
    mergeArray: () => (_, source) => {
        return source;
    }
});
// safe json parse
function parseJSONSafe(possibleJSON) {
    try {
        return (0, secure_json_parse_1.default)(possibleJSON);
    }
    catch (e) {
        return possibleJSON;
    }
}
// make camelCase keys
const camelCaseKeys = (camelize) => function processKeys(obj, value, key) {
    const camelized = camelize ? key : (0, camelcase_1.default)(key);
    if (value == null) {
        obj[camelized] = value;
        return obj;
    }
    if (typeof value === 'object') {
        const gatherer = obj[camelized] = Object.create(null);
        for (const [innerKey, innerValue] of Object.entries(value)) {
            processKeys(gatherer, innerValue, innerKey);
        }
    }
    else if (typeof value === 'string') {
        obj[camelized] = parseJSONSafe(value);
    }
    else {
        obj[camelized] = value;
    }
    return obj;
};
// read file from path and try to parse it
const mergeFileFactory = (crashOnError) => {
    return async (configuration, absPath) => {
        (0, node_assert_1.strict)(path_1.default.isAbsolute(absPath), `${absPath} must be an absolute path`);
        try {
            debug('loading %s', absPath);
            const data = absPath.endsWith('.json')
                ? (0, secure_json_parse_1.default)(await promises_1.default.readFile(absPath))
                : await import(absPath).then(m => m.default || m);
            debug('loaded', data);
            return merge(configuration, data);
        }
        catch (err) {
            if (crashOnError) {
                throw err;
            }
            else {
                process.stderr.write(`Failed to include file ${absPath}, err: ${err.message}\n`);
                return configuration;
            }
        }
    };
};
function possibleJSONStringToArray(filePaths) {
    let files;
    try {
        files = JSON.parse(filePaths);
    }
    catch (e) {
        files = [filePaths];
    }
    if (!isArray(files)) {
        throw new Error('NCONF_FILE_PATH must be a stringified array or a string');
    }
    return files;
}
exports.possibleJSONStringToArray = possibleJSONStringToArray;
async function resolveAbsPaths(paths) {
    debug('resolving abs paths for %j', paths);
    const absolutePaths = new Set();
    for (const filePath of paths) {
        debug('evaluating %s', filePath);
        const stats = await promises_1.default.stat(filePath);
        if (stats.isFile()) {
            debug('isFile');
            absolutePaths.add(require.resolve(filePath));
            debug('file added', require.resolve(filePath));
        }
        else if (stats.isDirectory()) {
            // NOTE: can be improved
            // this is an extra call, but we dont care since it's a one-time op
            const absPaths = await (0, glob_1.glob)(`${filePath}/*.{ts,cts,mts,js,cjs,mjs,json}`, {
                absolute: true,
                ignore: '/**/*.d.ts'
            });
            debug('isDir - %s - %j', filePath, absPaths);
            for (const path of absPaths) {
                absolutePaths.add(path);
            }
        }
        else {
            debug('ignoring %s', filePath);
        }
    }
    debug('prepared abs paths set - %j', Array.from(absolutePaths));
    return Array.from(absolutePaths);
}
async function globFiles(prependFile, filePaths = [], configuration = Object.create(null), crashOnError = true) {
    // if we get parsed JSON array - use it right away
    const files = isArray(filePaths)
        ? filePaths
        : possibleJSONStringToArray(filePaths);
    if (typeof prependFile === 'string') {
        files.unshift(prependFile);
    }
    // prepare merger
    const mergeFile = mergeFileFactory(crashOnError);
    // resolve paths and merge
    for (const file of await resolveAbsPaths(files)) {
        debug('merging config for %s', file);
        configuration = await mergeFile(configuration, file);
        debug('merged - prepared %j', configuration);
    }
    return configuration;
}
exports.globFiles = globFiles;
async function loadConfiguration(crashOnError, prependFile, appendConfiguration) {
    // load dotenv
    const dotenvConfig = {
        verbose,
        encoding: env.DOTENV_ENCODING || 'utf-8',
        path: env.DOTENV_FILE_PATH || `${cwd}/.env`,
    };
    // load dotenv and report error
    const result = dotenv_1.default.config(dotenvConfig);
    if (result.error) {
        debug('failed to load %s due to', dotenvConfig.path, result.error);
    }
    // do we camelize?
    const camelize = hasOwnProperty.call(env, 'NCONF_NO_CAMELCASE');
    const namespaceKey = env.NCONF_NAMESPACE;
    const filePaths = env.NCONF_FILE_PATH;
    // if we don't have it yet
    (0, node_assert_1.strict)(namespaceKey, 'NCONF_NAMESPACE must be present in order to parse your configuration');
    // init nconf store
    nconf_1.default.use('memory');
    nconf_1.default.reset();
    nconf_1.default.argv();
    nconf_1.default.env({
        separator: env.NCONF_SEPARATOR || '__',
        match: hasOwnProperty.call(env, 'NCONF_MATCH') ? new RegExp(env.NCONF_MATCH, env.NCONF_MATCH_OPTS) : null,
        whitelist: hasOwnProperty.call(env, 'NCONF_WHITELIST') ? JSON.parse(env.NCONF_WHITELIST) : null,
    });
    // pull camelCase data
    const namespace = nconf_1.default.get(namespaceKey);
    const configFromEnv = Object.create(null);
    // if there is anything for recording
    debug('retrieved namespace', namespace);
    if (namespace) {
        (0, node_assert_1.strict)(typeof namespace === 'object' && namespace !== null && !Array.isArray(namespace), 'namespace must be a js object');
        const normalizer = camelCaseKeys(camelize);
        for (const [key, value] of Object.entries(namespace)) {
            normalizer(configFromEnv, value, key);
        }
    }
    debug('normalized config - %j', configFromEnv);
    let config = Object.create(null);
    if (filePaths || prependFile) {
        // nconf file does not merge configuration, it will either omit it
        // or overwrite it, since it's JSON and is already parsed, what we will
        // do is pass it into configuration as is after it was already camelCased and merged
        //
        // nconf.file(env.NCONF_FILE_PATH);
        debug('globbing files', prependFile, filePaths);
        config = await globFiles(prependFile, filePaths, config, crashOnError);
        debug('result\n%j', config);
    }
    debug('merging env\n%j', configFromEnv);
    config = merge(config, configFromEnv);
    debug('result\n%j', config);
    if (appendConfiguration !== undefined) {
        debug('appending config: %j', appendConfiguration);
        config = merge(config, appendConfiguration);
        debug('result: %j', config);
    }
    debug('prepared config\n%j', config);
    return config;
}
exports.loadConfiguration = loadConfiguration;
//# sourceMappingURL=load-config.js.map